@@ -13,7 +13,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */
 
 #include <cstdio>
@@ -23,60 +23,11 @@
 #include <linux/cdrom.h>
 #include <sys/ioctl.h>
 #include <unistd.h>
+#include "msf_time.h"
 #include "cd.h"
 
 
-Msf_time::Msf_time( int m, int s, int f ) throw()
-  : _minute( m ), _second( s ), _frame( f )
-  {
-  if( _frame >= fps ) { _second += _frame / fps; _frame %= fps; }
-  else if( _frame < 0 )
-    { int i = (fps - 1 - _frame) / fps; _second -= i; _frame += fps * i; }
-
-  if( _second >= 60 ) { _minute += _second / 60; _second %= 60; }
-  else if( _second < 0 )
-    { int i = (59 - _second) / 60; _minute -= i; _second += 60 * i; }
-
-  if( _minute < 0 ) _minute = _second = _frame = 0;
-  }
-
-
-Msf_time & Msf_time::operator+=( const Msf_time & a ) throw()
-  {
-  _frame += a._frame; if( _frame >= fps ) { _frame -= fps; ++_second; }
-  _second += a._second; if( _second >= 60 ) { _second -= 60; ++_minute; }
-  _minute += a._minute;
-  return *this;
-  }
-
-
-Msf_time & Msf_time::operator-=( const Msf_time & a ) throw()
-  {
-  _frame -= a._frame; if( _frame < 0 ) { _frame += fps; --_second; }
-  _second -= a._second; if( _second < 0 ) { _second += 60; --_minute; }
-  _minute -= a._minute;
-  if( _minute < 0 ) _minute = _second = _frame = 0;
-  return *this;
-  }
-
-
-bool Msf_time::operator<( const Msf_time & a ) const throw()
-  {
-  return _minute < a._minute ||
-      ( _minute == a._minute && _second < a._second ) ||
-      ( _minute == a._minute && _second == a._second && _frame < a._frame );
-  }
-
-
-bool Msf_time::operator>( const Msf_time & a ) const throw()
-  {
-  return _minute > a._minute ||
-      ( _minute == a._minute && _second > a._second ) ||
-      ( _minute == a._minute && _second == a._second && _frame > a._frame );
-  }
-
-
-CD::CD( const char *filename ) throw()
+CD::CD( const char * filename ) throw()
   : status( no_disc ),
     _track( 0 ), _first_track( 0 ), _last_track( 0 ), _index( 0 ), _loop( 0 ),
     _linear( true )
@@ -84,7 +35,8 @@ CD::CD( const char *filename ) throw()
   filedes = ::open( filename, O_RDONLY | O_NONBLOCK );
   if( filedes < 0 )
     {
-    fprintf( stderr, "Cannot open %s in non-blocking mode.\n", filename );
+    fprintf( stderr, "adcd: cannot open '%s' in non-blocking mode.\n", filename );
+    fprintf( stderr, "Please, verify if you have permission to read device '%s'.\n", filename );
     exit(1);
     }
   read_status();
@@ -94,13 +46,12 @@ CD::CD( const char *filename ) throw()
 CD::~CD() throw() { if ( filedes >= 0 ) ::close( filedes ); }
 
 
-bool CD::read_status( bool force ) throw()
+bool CD::read_status( const bool force ) throw()
   {
   if( !force && ( status == no_disc || status == stopped ) ) return false;
   const Status old_status = status;
   const int old_tracks = tracks();
-  int ds = ioctl( filedes, CDROM_DRIVE_STATUS );
-  switch( ds )
+  switch( ioctl( filedes, CDROM_DRIVE_STATUS ) )
     {
     case CDS_NO_INFO:
     case CDS_DISC_OK:
@@ -131,12 +82,12 @@ bool CD::read_status( bool force ) throw()
               if( _track < _first_track ) _track = _first_track;
               else if( _track > _last_track ) _track = _last_track;
               _time_abs = time_start( _track ); _time_rel = 0;
-              status = stopped; break;
+              status = stopped;
               }
             }
           }
         }
-      else break;
+      break;
     case CDS_NO_DISC:
     case CDS_TRAY_OPEN:
     case CDS_DRIVE_NOT_READY:
@@ -163,8 +114,10 @@ bool CD::read_status( bool force ) throw()
       case CDROM_AUDIO_INVALID: if( status != playing ) break;
       case CDROM_AUDIO_COMPLETED:
         if( status == playing )
-          { if( _loop == 2 || _time_rel <= Msf_time( 0, 2 ) ) play();
-          else if( !next_track() ) status = stopped; }
+          {
+          if( _loop == 2 || ( _linear && _time_rel <= Msf_time( 0, 2 ) ) ) play();
+          else if( !next_track() ) status = stopped;
+          }
         break;
       case CDROM_AUDIO_ERROR: status = stopped; break;
       case CDROM_AUDIO_NO_STATUS: break;
@@ -212,7 +165,7 @@ void CD::play() throw()
   else if( _linear ) { ti.cdti_trk0 = _track; ti.cdti_trk1 = _last_track; }
   else if( _playlist.size() )
     {
-    if( _index >= _playlist.size() ) _index = _playlist.size() - 1;
+    if( _index >= (int)_playlist.size() ) _index = _playlist.size() - 1;
     ti.cdti_trk0 = ti.cdti_trk1 = _track = _playlist[_index];
     }
   else return;
@@ -239,7 +192,7 @@ bool CD::next_track() throw()
     }
   else if( _playlist.size() )
     {
-    if( _index < _playlist.size() - 1 ) _track = _playlist[++_index];
+    if( _index < (int)_playlist.size() - 1 ) _track = _playlist[++_index];
     else if( _loop ) { _index = 0; _track = _playlist[_index]; }
     else return false;
     }
@@ -268,7 +221,7 @@ bool CD::prev_track() throw()
   }
 
 
-bool CD::seek_forward( int seconds ) throw()
+bool CD::seek_forward( const int seconds ) throw()
   {
   if( status != playing || !_timelist.size() ||
       ( !_linear && !_playlist.size() ) ) return false;
@@ -294,7 +247,7 @@ bool CD::seek_forward( int seconds ) throw()
   }
 
 
-bool CD::seek_backward( int seconds ) throw()
+bool CD::seek_backward( const int seconds ) throw()
   {
   if( status != playing || !_timelist.size() ||
       ( !_linear && !_playlist.size() ) ) return false;
@@ -323,21 +276,21 @@ bool CD::seek_backward( int seconds ) throw()
   }
 
 
-void CD::loop( int new_loop ) throw()
+void CD::loop( const int new_loop ) throw()
   {
   _loop = (new_loop < 0) ? 0 : new_loop % 3;
   if( _loop == 2 && _linear && status == playing ) seek_forward( 1 );
   }
 
 
-char * CD::loop_name() const throw()
+const char * CD::loop_name() const throw()
   {
   switch( _loop )
     {
-    case 0 : return "     ";
-    case 1 : return "Loop ";
-    case 2 : return "Loop1";
-    default: return "Loop?";
+    case 0 : return "No   ";
+    case 1 : return "Disc ";
+    case 2 : return "Track";
+    default: return "error";
     }
   }
 
@@ -346,11 +299,11 @@ void CD::playlist( const std::vector< int > & pl ) throw()
   {
   _playlist = pl;
   if( pl.size() == 0 ) _index = 0;
-  else if( _index >= pl.size() ) _index = pl.size() - 1;
+  else if( _index >= (int)pl.size() ) _index = pl.size() - 1;
   }
 
 
-char * CD::status_name() const throw()
+const char * CD::status_name() const throw()
   {
   switch( status )
     {
@@ -363,7 +316,7 @@ char * CD::status_name() const throw()
   }
 
 
-bool CD::track( int new_track, bool start ) throw()
+bool CD::track( const int new_track, const bool start ) throw()
   {
   if( start )
     {
@@ -376,10 +329,10 @@ bool CD::track( int new_track, bool start ) throw()
   _track = new_track;
   if( !_linear && _playlist.size() && _playlist[_index] != _track )
     {
-    for( size_t i = _index + 1; i < _playlist.size(); ++i )
+    for( unsigned int i = _index + 1; i < _playlist.size(); ++i )
       if( _track == _playlist[i] ) { _index = i; break; }
     if( _playlist[_index] != _track )
-      for( size_t i = 0; i < _playlist.size(); ++i )
+      for( unsigned int i = 0; i < _playlist.size(); ++i )
         if( _track == _playlist[i] ) { _index = i; break; }
     if( _playlist[_index] != _track ) _linear = true;
     }
@@ -408,7 +361,7 @@ bool CD::volume( int vol ) const throw()
   }
 
 
-Msf_time CD::time( Time_mode mode ) const throw()
+Msf_time CD::time( const Time_mode mode ) const throw()
   {
   if( _timelist.size() ) switch( mode )
     {
@@ -421,7 +374,7 @@ Msf_time CD::time( Time_mode mode ) const throw()
   }
 
 
-Msf_time CD::time_start( int track ) const throw()
+Msf_time CD::time_start( const int track ) const throw()
   {
   if( _timelist.size() && tracks() &&
       track >= _first_track && track <= _last_track )
@@ -430,10 +383,27 @@ Msf_time CD::time_start( int track ) const throw()
   }
 
 
-Msf_time CD::time_end( int track ) const throw()
+Msf_time CD::time_end( const int track ) const throw()
   {
   if( _timelist.size() && tracks() &&
       track >= _first_track && track <= _last_track )
     return _timelist[track-_first_track].end;
   return 0;
   }
+
+
+Msf_time CD::time_track( const int track ) const throw()
+  {
+  if( _timelist.size() && tracks() &&
+      track >= _first_track && track <= _last_track )
+    return _timelist[track-_first_track].end - _timelist[track-_first_track].start;
+  return 0;
+  }
+
+
+Msf_time CD::time_disc() const throw()
+  {
+  if( _timelist.size() )
+    return _timelist.back().end - _timelist.front().start;
+  return 0;
+  }
