@@ -13,35 +13,9 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */
 
-
-class Msf_time			// Non negative time (instant or duration)
-  {
-  static const int fps = 75;	// Frames per second
-  int _minute, _second, _frame;
-
-public:
-  Msf_time( int m = 0, int s = 0, int f = 0 ) throw();
-
-  Msf_time & operator+=( const Msf_time & a ) throw();
-  Msf_time & operator-=( const Msf_time & a ) throw();
-  Msf_time operator+( const Msf_time & a ) const throw()
-    { Msf_time r( *this ); r += a; return r; }
-  Msf_time operator-( const Msf_time & a ) const throw()
-    { Msf_time r( *this ); r -= a; return r; }
-  bool operator<( const Msf_time & a ) const throw();
-  bool operator>( const Msf_time & a ) const throw();
-  bool operator<=( const Msf_time & a ) const throw() { return !(*this > a); }
-  bool operator>=( const Msf_time & a ) const throw() { return !(*this < a); }
-
-  int minute() const throw() { return _minute; }
-  int second() const throw() { return _second; }
-  int frame() const throw() { return _frame; }
-  };
-
-
 class CD
   {
   enum Status { no_disc, stopped, paused, playing };
@@ -53,17 +27,17 @@ class CD
   int _track;			// current track
   int _first_track;		// first track on current disc (not always 1)
   int _last_track;		// last track on current disc
-  size_t _index;		// index in playlist
-  int _loop;			// Loop mode (0=no_loop, 1=Loop, 2=Loop1)
-  bool _linear;			// Linear/Playlist mode
+  int _index;			// index in playlist
+  int _loop;			// Loop mode (0=no_loop, 1=Disc, 2=Track)
+  bool _linear;			// Linear / Playlist mode
   std::vector< int > _playlist;
   std::vector< Track_times > _timelist;
 
 public:
-  CD( const char *filename = "/dev/cdrom" ) throw();
+  CD( const char * filename = "/dev/cdrom" ) throw();
   ~CD() throw();
 
-  bool read_status( bool force = true ) throw();
+  bool read_status( const bool force = true ) throw();
 
   void close() throw();
   void open() throw();
@@ -73,23 +47,23 @@ public:
 
   bool next_track() throw();
   bool prev_track() throw();
-  bool seek_forward( int seconds ) throw();
-  bool seek_backward( int seconds ) throw();
+  bool seek_forward( const int seconds ) throw();
+  bool seek_backward( const int seconds ) throw();
 
   int  index() const throw() { return _index; }
   bool linear() const throw() { return _linear; }
-  void linear( bool new_linear ) throw() { _linear = new_linear; }
+  void linear( const bool new_linear ) throw() { _linear = new_linear; }
   int loop() const throw() { return _loop; }
-  void loop( int new_loop ) throw();
-  char * loop_name() const throw();
+  void loop( const int new_loop ) throw();
+  const char * loop_name() const throw();
   const std::vector< int > & playlist() const throw() { return _playlist; }
   void playlist( const std::vector< int > & pl ) throw();
-  char * status_name() const throw();
+  const char * status_name() const throw();
 
   int  first_track() const throw() { return _first_track; }
   int  last_track() const throw() { return _last_track; }
   int  track() const throw() { return _track; }
-  bool track( int new_track, bool start = false ) throw();
+  bool track( const int new_track, const bool start = false ) throw();
   int  tracks() const throw()
     { return (_last_track > 0) ? _last_track - _first_track + 1 : 0; }
 
@@ -97,7 +71,9 @@ public:
   bool volume( int vol ) const throw();
 
   enum Time_mode { relative, rem_rel, absolute, rem_abs };
-  Msf_time time( Time_mode mode ) const throw();
-  Msf_time time_start( int track ) const throw();
-  Msf_time time_end( int track ) const throw();
+  Msf_time time( const Time_mode mode ) const throw();
+  Msf_time time_start( const int track ) const throw();
+  Msf_time time_end( const int track ) const throw();
+  Msf_time time_track( const int track ) const throw();
+  Msf_time time_disc() const throw();
   };
